      subroutine check_zground_consistency(Nz,Ntheta,Nphi,z,ok)
      implicit none
      include 'max.inc'
c     
c     Purpose: to check consistency between ground altitudes
c     The code will not work properly if any ground altitude is above the altitude at the top
c     of the first layer for any adjacent column
c
c     Inputs:
c       + Nz,Ntheta,Nphi: number of cells
c       + z: altitudes
c
c     Output
c       +ok: 1 if radiuses satisfy dependances; 0 otherwise
c

c     inputs
      integer Nz(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision z(0:Nr_mx,1:Ntheta_mx,1:Nphi_mx)
c     output
      integer ok
c     temp
      integer ir,itheta,iphi
      integer itheta_prev,itheta_next
      integer iphi_prev,iphi_next,iphi_mod
      integer strlen
      character*100 label
      label='check_zground_consistency'

      ok=1
      do itheta=2,Ntheta-1
         do iphi=1,Nphi
            call prev_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           itheta_prev,iphi_mod)
            call prev_phi_cell(Nphi,iphi,iphi_prev)
            call next_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           itheta_next,iphi_mod)
            call next_phi_cell(Nphi,iphi,iphi_next)
c     itheta+1,iphi-1
            if (z(0,itheta,iphi).gt.z(1,itheta_next,iphi_prev)) then
               write(*,*) 'Error from routine ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
               write(*,*) 'is above z(1,',itheta_next,',',iphi_prev,
     &              ')=',z(1,itheta_next,iphi_prev)
               ok=0
               goto 666
            endif
c     itheta+1,iphi
            if (z(0,itheta,iphi).gt.z(1,itheta_next,iphi)) then
               write(*,*) 'Error from routine ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
               write(*,*) 'is above z(1,',itheta_next,',',iphi,
     &              ')=',z(1,itheta_next,iphi)
               ok=0
               goto 666
            endif
c     itheta+1,iphi+1
            if (z(0,itheta,iphi).gt.z(1,itheta_next,iphi_next)) then
               write(*,*) 'Error from routine ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
               write(*,*) 'is above z(1,',itheta_next,',',iphi_next,
     &              ')=',z(1,itheta_next,iphi_next)
               ok=0
               goto 666
            endif

c     itheta,iphi-1
            if (z(0,itheta,iphi).gt.z(1,itheta,iphi_prev)) then
               write(*,*) 'Error from routine ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
               write(*,*) 'is above z(1,',itheta,',',iphi_prev,
     &              ')=',z(1,itheta,iphi_prev)
               ok=0
               goto 666
            endif
c     itheta,iphi+1
            if (z(0,itheta,iphi).gt.z(1,itheta,iphi_next)) then
               write(*,*) 'Error from routine ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
               write(*,*) 'is above z(1,',itheta,',',iphi_next,
     &              ')=',z(1,itheta,iphi_next)
               ok=0
               goto 666
            endif

c     itheta-1,iphi-1
            if (z(0,itheta,iphi).gt.z(1,itheta_prev,iphi_prev)) then
               write(*,*) 'Error from routine ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
               write(*,*) 'is above z(1,',itheta_prev,',',iphi_prev,
     &              ')=',z(1,itheta_prev,iphi_prev)
               ok=0
               goto 666
            endif
c     itheta-1,iphi
            if (z(0,itheta,iphi).gt.z(1,itheta_prev,iphi)) then
               write(*,*) 'Error from routine ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
               write(*,*) 'is above z(1,',itheta_prev,',',iphi,
     &              ')=',z(1,itheta_prev,iphi)
               ok=0
               goto 666
            endif
c     itheta-1,iphi+1
            if (z(0,itheta,iphi).gt.z(1,itheta_prev,iphi_next)) then
               write(*,*) 'Error from routine ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
               write(*,*) 'is above z(1,',itheta_prev,',',iphi_next,
     &              ')=',z(1,itheta_prev,iphi_next)
               ok=0
               goto 666
            endif

         enddo ! iphi
      enddo ! itheta

c     south pole
      itheta=1
      do iphi=1,Nphi
         call prev_phi_cell(Nphi,iphi,iphi_prev)
         call next_phi_cell(Nphi,iphi,iphi_next)
         if (z(0,itheta,iphi).gt.z(1,itheta,iphi_prev)) then
            write(*,*) 'Error from routine ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
            write(*,*) 'is above z(1,',itheta,',',iphi_prev,
     &           ')=',z(1,itheta,iphi_prev)
            ok=0
            goto 666
         endif
         if (z(0,itheta,iphi).gt.z(1,itheta,iphi_next)) then
            write(*,*) 'Error from routine ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
            write(*,*) 'is above z(1,',itheta,',',iphi_next,
     &           ')=',z(1,itheta,iphi_next)
            ok=0
            goto 666
         endif
      enddo ! iphi

c     north pole
      itheta=Ntheta
      do iphi=1,Nphi
         call prev_phi_cell(Nphi,iphi,iphi_prev)
         call next_phi_cell(Nphi,iphi,iphi_next)
         if (z(0,itheta,iphi).gt.z(1,itheta,iphi_prev)) then
            write(*,*) 'Error from routine ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
            write(*,*) 'is above z(1,',itheta,',',iphi_prev,
     &           ')=',z(1,itheta,iphi_prev)
            ok=0
            goto 666
         endif
         if (z(0,itheta,iphi).gt.z(1,itheta,iphi_next)) then
            write(*,*) 'Error from routine ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'z(0,',itheta,',',iphi,')=',z(0,itheta,iphi)
            write(*,*) 'is above z(1,',itheta,',',iphi_next,
     &           ')=',z(1,itheta,iphi_next)
            ok=0
            goto 666
         endif
      enddo ! iphi

 666  continue
      return
      end



      subroutine prev_theta_cell(Ntheta,Nphi,itheta,iphi,
     &     itheta_prev,iphi_mod)
      implicit none
c     purpose: to return 'itheta_prev', the index of the previous
c     cell on a latitude row

c     inputs
      integer Ntheta,Nphi,itheta,iphi
c     outputs
      integer itheta_prev,iphi_mod
c     temp
      integer strlen
      character*100 label
      label='prev_theta_cell'

      if ((itheta.le.0).or.(itheta.ge.Ntheta+1)) then
         write(*,*) 'Error from routine ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for itheta=',itheta
         stop
      endif
      
      if (itheta.eq.1) then
         itheta_prev=1
         if (iphi.le.Nphi/2) then
            iphi_mod=iphi+Nphi/2
         else
            iphi_mod=iphi-Nphi/2
         endif
      else
         itheta_prev=itheta-1
      endif

      return
      end



      subroutine next_theta_cell(Ntheta,Nphi,itheta,iphi,
     &     itheta_next,iphi_mod)
      implicit none
c     purpose: to return 'itheta_next', the index of the next
c     cell on a latitude row
      integer strlen
      integer Ntheta,Nphi,itheta,iphi,itheta_next,iphi_mod
      character*100 label
      label='next_theta_cell'

      if ((itheta.le.0).or.(itheta.ge.Ntheta+1)) then
         write(*,*) 'Error from routine ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for itheta=',itheta
         stop
      endif
      
      if (itheta.eq.Ntheta) then
         itheta_next=Ntheta
         if (iphi.le.Nphi/2) then
            iphi_mod=iphi+Nphi/2
         else
            iphi_mod=iphi-Nphi/2
         endif
      else
         itheta_next=itheta+1
      endif

      return
      end



      subroutine prev_phi_cell(Nphi,iphi,iphi_prev)
      implicit none
c     purpose: to return 'iphi_prev', the index of the previous
c     cell on a longitude row
      integer strlen
      integer Nphi,iphi,iphi_prev
      character*100 label
      label='prev_phi_cell'

      if ((iphi.le.0).or.(iphi.ge.Nphi+1)) then
         write(*,*) 'Error from routine ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for iphi=',iphi
         write(*,*) 'Nphi=',Nphi
         stop
      endif
      
      if (iphi.eq.1) then
         iphi_prev=Nphi
      else
         iphi_prev=iphi-1
      endif

      return
      end



      subroutine next_phi_cell(Nphi,iphi,iphi_next)
      implicit none
c     purpose: to return 'iphi_next', the index of the next
c     cell on a longitude row
      integer strlen
      integer Nphi,iphi,iphi_next
      character*100 label
      label='next_phi_cell'

      if ((iphi.le.0).or.(iphi.ge.Nphi+1)) then
         write(*,*) 'Error from routine ',label(1:strlen(label)),' :'
         write(*,*) 'wrong value for iphi=',iphi
         stop
      endif
      
      if (iphi.eq.Nphi) then
         iphi_next=1
      else
         iphi_next=iphi+1
      endif

      return
      end
