c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine pdf_tau_abs(gen,tau_max,tau_in,tau_out,pdf)
      implicit none
c     
c     Purpose: to compute the pdf of an absorption optical depth OR to generate an optical
c     depth according to the absorption pdf
c
c     Inputs 
c       + gen:
c         - gen=0: only compute the pdf of "tau_in" within the [0:tau_max] range
c         - gen=1: generate "tau_out" within the [0:tau_max] range
c       + tau_max: higher limit of the absorption optical depth variation range
c       + tau_in: value of the optical depth the pdf has to be computed for, if gen=0
c
c     Outputs :
c       + tau_out: value of the optical depth that was generated if gen=1
c       + pdf: value of the absorption pdf (gen=0 or gen=1)
c

c     inputs
      integer gen
      double precision tau_max,tau_in
c     outputs
      double precision tau_out,pdf
c     temp
      double precision R

      if (gen.eq.1) then
         call random_gen(R)
         tau_out=-dlog(1.D+0-R*(1.D+0-dexp(-tau_max)))
         tau_in=tau_out
      endif
      pdf=dexp(-tau_in)/(1.D+0-dexp(-tau_max))

      return
      end
