c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine identify_clouds(Nclouds,clouds_coord,r1,r2,
     &     theta1,theta2,phi1,phi2,Nc,cindex,ccoord)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify clouds that intersect with the current box
c
c     Inputs:
c       + Nclouds: total number of clouds in the planetary atmosphere
c       + clouds_coord: (r,theta,phi) limits of each cloud
c       + r1,r2,theta1,theta2,phi1,phi2: limits of the current box
c
c     Outputs:
c       + Nc: number of clouds that intersect with the current box
c       + cindex: indexes of the clouds that intersect with the box
c       + ccoord: (r,theta,phi) limits of each cloud that intersect
c                with the current box
c

c     input
      integer Nclouds
      double precision clouds_coord(1:Nclouds_mx,1:6)
      double precision r1,r2,theta1,theta2,phi1,phi2
c     output
      integer Nc
      integer cindex(1:Nclouds_mx)
      double precision ccoord(1:Nclouds_mx,1:6)
c     temp
      integer cloud,i
      integer intersect(1:Nclouds_mx)

      Nc=0
      do cloud=1,Nclouds
         call cloud_intersects(r1,r2,theta1,theta2,phi1,phi2,
     &        clouds_coord(cloud,1),clouds_coord(cloud,2),
     &        clouds_coord(cloud,3),clouds_coord(cloud,4),
     &        clouds_coord(cloud,5),clouds_coord(cloud,6),
     &        intersect(cloud))
         if (intersect(cloud).eq.1) then
            Nc=Nc+1
            cindex(Nc)=cloud
            do i=1,6
               ccoord(Nc,i)=clouds_coord(cloud,i)
            enddo
         endif
      enddo

      return
      end



      subroutine cloud_intersects(r1,r2,theta1,theta2,phi1,phi2,
     &     r1c,r2c,theta1c,theta2c,phi1c,phi2c,intersect)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify whether or not the provided cloud
c     and box intersect
c
c     Inputs:
c       + r1,r2,theta1,theta2,phi1,phi2: limits of the box
c       + r1c,r2c,theta1c,theta2c,phi1c,phi2c: limits of the cloud
c
c     Outputs:
c       + intersect:
c         - intersect=0: cloud and box do NOT intersect
c         - intersect=1: cloud and box do intersect
c

c     input
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision r1c,r2c,theta1c,theta2c,phi1c,phi2c
c     output
      integer intersect
c     temp
      integer corner
      double precision coord(1:Nmat_mx)
      integer in_box(1:8)
      integer in_cloud(1:8)
      character*100 label

      label='subroutine cloud_intersects'
      call test_box(r1,r2,theta1,theta2,phi1,phi2,label)
      call test_box(r1c,r2c,theta1c,theta2c,phi1c,phi2c,label)
      
      intersect=0 ! default
c     testing cloud corners
      do corner=1,8
         call corner_coordinates(r1c,r2c,theta1c,theta2c,phi1c,phi2c,
     &        corner,coord)
         call is_in_box(coord,r1,r2,theta1,theta2,
     &     phi1,phi2,in_box(corner))
c     if only one corner of the cloud is within the box
         if (in_box(corner).eq.1) then
c     then cloud and box intersect
            intersect=1
            goto 666
         endif
      enddo

c     testing box corners
      do corner=1,8
         call corner_coordinates(r1,r2,theta1,theta2,phi1,phi2,
     &        corner,coord)
         call is_in_box(coord,r1c,r2c,theta1c,theta2c,
     &        phi1c,phi2c,in_cloud(corner))
c     if only one corner of the box is within the cloud
         if (in_cloud(corner).eq.1) then
c     then cloud and box intersect
            intersect=1
            goto 666
         endif
      enddo

c     testing r-section
      if ((theta1c.ge.theta1).and.(theta2c.le.theta2).and.
     &     (phi1c.ge.phi1).and.(phi2c.le.phi2)) then
         intersect=1
         goto 666
      endif
c     testing theta-section
      if ((r1c.ge.r1).and.(r2c.le.r2).and.
     &     (phi1c.ge.phi1).and.(phi2c.le.phi2)) then
         intersect=1
         goto 666
      endif
c     testing phi-section
      if ((r1c.ge.r1).and.(r2c.le.r2).and.
     &     (theta1c.ge.theta1).and.(theta2c.le.theta2)) then
         intersect=1
         goto 666
      endif

 666  continue
      return
      end


      
      subroutine corner_coordinates(r1,r2,theta1,theta2,phi1,phi2,
     &     corner,coord)
      implicit none
      include 'max.inc'
c
c     Purpose: to return (r,theta,phi) spherical coordinates
c     of the required box corner
c
c     Inputs:
c       + r1,r2,theta1,theta2,phi1,phi2: limits of the box
c       + corner: corner index (1-8)
c
c     Outputs:
c       + coord: (r,theta,phi) spherical coordinates of the corner
c

c     input
      double precision r1,r2,theta1,theta2,phi1,phi2
      integer corner
c     output
      double precision coord(1:Nmat_mx)
c     temp
      double precision r,theta,phi
      character*100 label

      label='subroutine corner_coordinates'
      call test_corner(corner,label)

      if (corner.eq.1) then
         r=r1
         theta=theta1
         phi=phi1
      else if (corner.eq.2) then
         r=r2
         theta=theta1
         phi=phi1
      else if (corner.eq.3) then
         r=r1
         theta=theta1
         phi=phi2
      else if (corner.eq.4) then
         r=r2
         theta=theta1
         phi=phi2
      else if (corner.eq.5) then
         r=r1
         theta=theta2
         phi=phi1
      else if (corner.eq.6) then
         r=r2
         theta=theta2
         phi=phi1
      else if (corner.eq.7) then
         r=r1
         theta=theta2
         phi=phi2
      else if (corner.eq.8) then
         r=r2
         theta=theta2
         phi=phi2
      endif
      coord(1)=r
      coord(2)=theta
      coord(3)=phi

      return
      end



      subroutine test_corner(corner,label)
      implicit none
      integer corner,strlen
      character*100 label

      if ((corner.lt.1).or.(corner.gt.8)) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Incorrect value for:'
         write(*,*) 'corner=',corner
         stop
      endif

      return
      end
