c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine look_for_errors(eps_st4,Nevent,
     &     nchunks,ref,intp,pf,vri,toa_ad,
     &     Radius,Tsun,Rsun,dist,theta0,phi0)
      implicit none
c
c     Purpose: to identify possible mistakes in "data.in" and "options.in"
c
      include 'max.inc'
      include 'param.inc'
      
c     data
      double precision eps_st4
      integer Nevent,nchunks
c     options
      integer ref,intp,pf,vri,toa_ad
c     planet data
      double precision Radius,Tsun,Rsun,dist,theta0,phi0

c     ---------------------------------------
c     Checking data
c     ---------------------------------------

      if (eps_st4.le.0.0D+0) then
         write(*,*) 'Incorrect value in "data.in":'
         write(*,*) 'Percentage of energy'
         stop
      endif
      if (Nevent.le.0) then
         write(*,*) 'Incorrect value in "data.in":'
         write(*,*) 'Number of statistical events'
         stop
      endif
      if ((nchunks.le.0).and.(nchunks.ne.-1)) then
         write(*,*) 'Incorrect value in "data.in":'
         write(*,*) 'Number of computational chunks'
         stop
      endif

c     ---------------------------------------
c     Checking options
c     ---------------------------------------

      if ((ref.ne.1).and.(ref.ne.2)) then
         write(*,*) 'Incorrect value in "options.in":'
         write(*,*) 'reflexion at ground level'
         stop
      endif
      if ((intp.ne.0).and.(intp.ne.1)) then
         write(*,*) 'Incorrect value in "options.in":'
         write(*,*) 'integrate Planck intensity'
         stop
      endif
      if ((pf.ne.1).and.(pf.ne.2)) then
         write(*,*) 'Incorrect value in "options.in":'
         write(*,*) 'phase function type for scattering by clouds'
         stop
      endif
      if ((vri.ne.0).and.(vri.ne.1)) then
         write(*,*) 'Incorrect value in "options.in":'
         write(*,*) 'take into account refraction effects'
         stop
      endif
      if ((toa_ad.ne.0).and.(toa_ad.ne.1)) then
         write(*,*) 'Incorrect value in "options.in":'
         write(*,*) 'computation of TOA fluxes angular distribution'
         stop
      endif

c     ---------------------------------------
c     Checking planetary data
c     ---------------------------------------

      if (Radius.le.0.0D+0) then
         write(*,*) 'Incorrect value in "planet.in":'
         write(*,*) 'Equatorial radius'
         stop
      endif
      if (Tsun.le.0.0D+0) then
         write(*,*) 'Incorrect value in "planet.in":'
         write(*,*) 'Temperature of the star'
         stop
      endif
      if (Rsun.le.0.0D+0) then
         write(*,*) 'Incorrect value in "planet.in":'
         write(*,*) 'Radius of the star'
         stop
      endif
      if (dist.le.0.0D+0) then
         write(*,*) 'Incorrect value in "planet.in":'
         write(*,*) 'Distance from planet to star'
         stop
      endif
      if ((theta0.lt.-pi/2.0D+0).or.(theta0.gt.pi/2.0D+0)) then
         write(*,*) 'Incorrect value in "planet.in":'
         write(*,*) 'latitude of the point where the star',
     &        ' is at the zenith'
         stop
      endif
      if ((phi0.lt.0.0D+0).or.(theta0.gt.2.0D+0*pi)) then
         write(*,*) 'Incorrect value in "planet.in":'
         write(*,*) 'longitude of the point where the star',
     &        ' is at the zenith'
         stop
      endif


      return
      end
