c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine solar_postcalc(dim,prog_name,Nevent,toa_ad,
     &     Nr,Ntheta,Nphi,Nb_gas,
     &     Radius,rspace,
     &     z,r,theta,phi,
     &     lambda_gas_lo,lambda_gas_hi,
     &     sum_flux_up,
     &     sum_flux_dn,
     &     sum_flux_net,
     &     sum_flux_toa,
     &     sum_budget,
     &     sum2_flux_up,
     &     sum2_flux_dn,
     &     sum2_flux_net,
     &     sum2_flux_toa,
     &     sum2_budget,
     &     solar_flux_up,
     &     solar_inc_flux_up,
     &     solar_flux_dn,
     &     solar_inc_flux_dn,
     &     solar_flux_net,
     &     solar_inc_flux_net,
     &     solar_flux_toa,
     &     solar_inc_flux_toa,
     &     solar_budget,
     &     solar_inc_budget,
     &     rground,phi0)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: compute final results from MC sums and sum of squares
c
c     Inputs:
c       + dim: dimension of vectors
c       + prog_name: label of the program that calls this routine
c       + Nevent: number of statistical realizations
c       + toa_ad: 1 if TOA fluxes computation is required; 0 otherwise
c       + Nr,Ntheta,Nphi: number of cells
c       + Nb_gas: number of gas spectral narrow bands
c       + Radius: radius of the planet (m)
c       + space: radius at space level (m)
c       + z,r,theta,phi: limits of cells
c       + lambda_gas_lo,lambda_gas_hi: gas spectral narrowband intervals limits
c       + sum_flux_up: sum of contribution for solar upward flux at each spherical interface (W)
c       + sum_flux_dn: sum of contribution for solar downard flux at each spherical interface (W)
c       + sum_flux_net: sum of contribution for solar net flux at each spherical interface (W)
c       + sum_flux_toa: sum of contribution for solar TOA flux in each column (W)
c       + sum_budget: sum of contribution for solar budget in each cell (W/m)
c       + sum2_flux_up: sum of squared contribution for solar upward flux at each spherical interface (W)
c       + sum2_flux_dn: sum of squared contribution for solar downard flux at each spherical interface (W)
c       + sum2_flux_net: sum of squared contribution for solar net flux at each spherical interface (W)
c       + sum2_flux_toa: sum of squared contribution for solar TOA flux in each column (W)
c       + sum2_budget: sum of squared contribution for solar budget in each cell (W/m)
c       + rground: radius at ground level (m)
c       + phi0: longitude that is at noon (rad)
c
c     Outputs:
c       + solar_flux_up: solar upward flux for each spherical interface (W/m)
c       + solar_inc_flux_up: uncertainty over solar upward flux for each spherical interface (W/m)
c       + solar_flux_dn: solar downward flux for each spherical interface (W/m)
c       + solar_inc_flux_dn: uncertainty over solar downward flux for each spherical interface (W/m)
c       + solar_flux_net: solar net flux for each spherical interface (W/m)
c       + solar_inc_flux_net: uncertainty over solar net flux for each spherical interface (W/m)
c       + solar_flux_toa: solar TOA flux in each column (W/m)
c       + solar_inc_flux_toa: uncertainty over solar TOA flux in each column (W/m)
c       + solar_budget: solar radiative budget for each cell (W/m)
c       + solar_inc_budget: uncertainty over solar radiative budget for each cell (W/m)
c

c     Input
      integer dim,Nevent,Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi,Nb_gas
      integer toa_ad
      double precision Radius,rspace
      character*100 prog_name
      double precision z(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      double precision sum_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision sum_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision sum_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision sum_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,Nb_gas_mx) ! (W/m)
      double precision sum_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision sum2_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision sum2_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision sum2_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision sum2_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,Nb_gas_mx) ! (W/m)
      double precision sum2_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision rground,phi0
c     Output
      double precision solar_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision solar_inc_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision solar_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision solar_inc_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision solar_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision solar_inc_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision solar_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,Nb_gas_mx) ! (W/m)
      double precision solar_inc_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,Nb_gas_mx) ! (W/m)
      double precision solar_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision solar_inc_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx) ! (W/m)
c     temp
      integer ir,itheta,iphi,u_itheta,u_iphi,band
      double precision sum_solar_flux_toa(1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision sum_inc_solar_flux_toa(1:Ntheta_mx,1:Nphi_mx) ! (W/m)
      double precision solar_var_flux_up ! (W/m)
      double precision solar_var_flux_dn ! (W/m)
      double precision solar_var_flux_net ! (W/m)
      double precision solar_var_flux_toa ! (W/m)
      double precision solar_var_budget ! (W/m)
      double precision surface_r,volume_box
      integer year,month,day
      character*100 date_str,time_str
      double precision P(1:Nmat_mx)
      double precision time
      integer hour,min,sec
      integer strlen
      character*100 label
      label='subroutine postcalc'

c     Debug
c            do itheta=1,Ntheta
c               do iphi=1,Nphi
c                  do u_itheta=1,Ntheta
c                     do u_iphi=1,Nphi
c                        if (sum_flux_toa(itheta,iphi,
c     &                       u_itheta,u_iphi).ne.0.0D+0) then
c                           write(*,*)
c                           write(*,*) itheta,iphi,u_itheta,u_iphi
c                           write(*,*) sum_flux_toa(itheta,iphi,
c     &                       u_itheta,u_iphi)
c                        endif
c                     enddo
c                  enddo
c               enddo
c            enddo
c     Debug

c     solar upward fluxes
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=0,Nr(itheta,iphi)
               call statis(Nevent,
     &              sum_flux_up(ir,itheta,iphi), ! W
     &              sum2_flux_up(ir,itheta,iphi), ! W
     &              solar_flux_up(ir,itheta,iphi), ! W
     &              solar_var_flux_up, ! W
     &              solar_inc_flux_up(ir,itheta,iphi)) ! W
               solar_flux_up(ir,itheta,iphi)=
     &              solar_flux_up(ir,itheta,iphi)/
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi)) ! W/m
c               solar_var_flux_up=
c     &              solar_var_flux_up/
c     &              surface_r(r(ir,itheta,iphi),
c     &              theta(itheta-1),theta(itheta),
c     &              phi(iphi-1),phi(iphi))**2.0D+0 ! (W/m)
               solar_inc_flux_up(ir,itheta,iphi)=
     &              solar_inc_flux_up(ir,itheta,iphi)/
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi)) ! W/m
            enddo               ! ir
         enddo                  ! iphi
      enddo                     ! itheta

c     solar downward fluxes
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=0,Nr(itheta,iphi)
               call statis(Nevent,
     &              sum_flux_dn(ir,itheta,iphi),
     &              sum2_flux_dn(ir,itheta,iphi),
     &              solar_flux_dn(ir,itheta,iphi),
     &              solar_var_flux_dn,
     &              solar_inc_flux_dn(ir,itheta,iphi))
               solar_flux_dn(ir,itheta,iphi)=
     &              solar_flux_dn(ir,itheta,iphi)/
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi)) ! W/m
c               solar_var_flux_dn=
c     &              solar_var_flux_dn/
c     &              surface_r(r(ir,itheta,iphi),
c     &              theta(itheta-1),theta(itheta),
c     &              phi(iphi-1),phi(iphi))**2.0D+0 ! (W/m)
               solar_inc_flux_dn(ir,itheta,iphi)=
     &              solar_inc_flux_dn(ir,itheta,iphi)/
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi)) ! W/m
            enddo               ! ir
         enddo                  ! iphi
      enddo                     ! itheta

c     solar net fluxes
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=0,Nr(itheta,iphi)
               call statis(Nevent,
     &              sum_flux_net(ir,itheta,iphi),
     &              sum2_flux_net(ir,itheta,iphi),
     &              solar_flux_net(ir,itheta,iphi),
     &              solar_var_flux_net,
     &              solar_inc_flux_net(ir,itheta,iphi))
               solar_flux_net(ir,itheta,iphi)=
     &              solar_flux_net(ir,itheta,iphi)/
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi)) ! W/m
c               solar_var_flux_net=
c     &              solar_var_flux_net/
c     &              surface_r(r(ir,itheta,iphi),
c     &              theta(itheta-1),theta(itheta),
c     &              phi(iphi-1),phi(iphi))**2.0D+0 ! (W/m)
               solar_inc_flux_net(ir,itheta,iphi)=
     &              solar_inc_flux_net(ir,itheta,iphi)/
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi)) ! W/m
            enddo               ! ir
         enddo                  ! iphi
      enddo                     ! itheta

      if (toa_ad.eq.1) then
c     solar TOA fluxes
         do itheta=1,Ntheta
            do iphi=1,Nphi
               do u_itheta=1,Ntheta
                  do u_iphi=1,Nphi
                     do band=1,Nb_gas
                        call statis(Nevent,
     &                       sum_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band), ! W
     &                       sum2_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band), ! W
     &                       solar_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band), ! W
     &                       solar_var_flux_toa, ! W
     &                       solar_inc_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)) ! W
                        solar_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)=
     &                       solar_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)/
     &                       surface_r(r(Nr(itheta,iphi),
     &                       itheta,iphi),
     &                       theta(itheta-1),theta(itheta),
     &                       phi(iphi-1),phi(iphi)) ! W/m
                        solar_inc_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)=
     &                       solar_inc_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)/
     &                       surface_r(r(Nr(itheta,iphi),
     &                       itheta,iphi),
     &                       theta(itheta-1),theta(itheta),
     &                       phi(iphi-1),phi(iphi)) ! W/m
                     enddo      ! band
                  enddo         ! u_iphi
               enddo            ! u_itheta
            enddo               ! iphi
         enddo                  ! itheta
c     Debug
c     sum of TOA fluxes
         do itheta=1,Ntheta
            do iphi=1,Nphi
               sum_solar_flux_toa(itheta,iphi)=0.0D+0
               sum_inc_solar_flux_toa(itheta,iphi)=0.0D+0
               do u_itheta=1,Ntheta
                  do u_iphi=1,Nphi
                     do band=1,Nb_gas
                        sum_solar_flux_toa(itheta,iphi)=
     &                       sum_solar_flux_toa(itheta,iphi)+
     &                       solar_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)
                        sum_inc_solar_flux_toa(itheta,iphi)=
     &                       sum_inc_solar_flux_toa(itheta,iphi)+
     &                       solar_inc_flux_toa(itheta,iphi,
     &                       u_itheta,u_iphi,band)
                     enddo
                  enddo
               enddo
            enddo
         enddo
c     Debug
      endif

c     solar budgets
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=0,Nr(itheta,iphi)+1
               call statis(Nevent,
     &              sum_budget(ir,itheta,iphi),
     &              sum2_budget(ir,itheta,iphi),
     &              solar_budget(ir,itheta,iphi),
     &              solar_var_budget,
     &              solar_inc_budget(ir,itheta,iphi))
               if ((ir.eq.0).or.(ir.eq.Nr(itheta,iphi)+1)) then
c     radiative budgets of ground and space are computed in W/m
                  solar_budget(ir,itheta,iphi)=
     &                 solar_budget(ir,itheta,iphi)/
     &                 surface_r(r(ir,itheta,iphi),
     &                 theta(itheta-1),theta(itheta),
     &                 phi(iphi-1),phi(iphi)) ! W/m
c                  solar_var_budget=
c     &                 solar_var_budget/
c     &                 surface_r(r(ir,itheta,iphi),
c     &                 theta(itheta-1),theta(itheta),
c     &                 phi(iphi-1),phi(iphi))**2.0D+0 ! (W/m)
                  solar_inc_budget(ir,itheta,iphi)=
     &                 solar_inc_budget(ir,itheta,iphi)/
     &                 surface_r(r(ir,itheta,iphi),
     &                 theta(itheta-1),theta(itheta),
     &                 phi(iphi-1),phi(iphi)) ! W/m
               else
c     radiative budgets of volumic cells are computed in W/m
                  solar_budget(ir,itheta,iphi)=
     &                 solar_budget(ir,itheta,iphi)/
     &                 volume_box(r(ir-1,itheta,iphi),
     &                 r(ir,itheta,iphi),
     &                 theta(itheta-1),theta(itheta),
     &                 phi(iphi-1),phi(iphi)) ! W/m
c                  solar_var_budget=
c     &                 solar_var_budget/
c     &                 volume_box(r(ir-1,itheta,iphi),
c     &                 r(ir,itheta,iphi),
c     &                 theta(itheta-1),theta(itheta),
c     &                 phi(iphi-1),phi(iphi))**2.0D+0 ! (W/m)
                  solar_inc_budget(ir,itheta,iphi)=
     &                 solar_inc_budget(ir,itheta,iphi)/
     &                 volume_box(r(ir-1,itheta,iphi),
     &                 r(ir,itheta,iphi),
     &                 theta(itheta-1),theta(itheta),
     &                 phi(iphi-1),phi(iphi)) ! W/m
               endif
            enddo               ! ir
         enddo                  ! iphi
      enddo                     ! itheta
      
c     -------------------------------------------------
c                  FILE RECORDS
c     -------------------------------------------------

c     get the date and compute date string
      call get_date(year,month,day)
      call date_string(year,month,day,date_str)

c     **** recording integrated solar upward fluxes ****
      open(11,file='./results/integrated_solar_upward_fluxes.txt')
      write(11,10) 'Integrated solar upward fluxes file'
      write(11,10) 'Number of statistical events used:'
      write(11,*) Nevent
      write(11,10) 'File created by program: ',
     &     prog_name(1:strlen(prog_name))
      write(11,10) 'Date of record:',date_str(1:strlen(date_str))
      write(11,10) 'Number of latitude bands:'
      write(11,*) Nphi
      write(11,10) 'Number of longitude bands:'
      write(11,*) Ntheta
      write(11,*)

      do iphi=1,Nphi
         write(11,10) 'Latitude band index:'
         write(11,*) iphi
         write(11,10) 'Minimum phi (rad):'
         write(11,*) phi(iphi-1)
         P(1)=rground
         P(2)=0.0D+0
         P(3)=phi(iphi-1)
         call local_time(dim,P,phi0,time,hour,min,sec)
         call time_string(hour,min,sec,time_str)
         write(11,10) 'Local time (hh:mm:ss) :',time_str
         write(11,10) 'Maximum phi (rad):'
         write(11,*) phi(iphi)
         P(3)=phi(iphi)
         call local_time(dim,P,phi0,time,hour,min,sec)
         call time_string(hour,min,sec,time_str)
         write(11,10) 'Local time (hh:mm:ss) :',time_str
         write(11,*)

         do itheta=1,Ntheta
            write(11,10) 'Longitude band index:'
            write(11,*) itheta
            write(11,10) 'Minimum theta (rad):'
            write(11,*) theta(itheta-1)
            write(11,10) 'Maximum theta (rad):'
            write(11,*) theta(itheta)
            write(11,10) 'Number of vertical interfaces:'
            write(11,*) Nr(itheta,iphi)+1
            write(11,10) 'altitudes of interfaces (m):'
            write(11,50) (z(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(11,10) 'radiuses of interfaces (m):'
            write(11,50) (r(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(11,10) 'Integrated solar upward fluxes (W/m):'
            write(11,50) (solar_flux_up(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(11,10) 'Uncertainties (W/m):'
            write(11,50) (solar_inc_flux_up(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
         enddo ! itheta
      enddo ! iphi
      close(11)


c     **** recording integrated solar downward fluxes ****
      open(12,file='./results/integrated_solar_downward_fluxes.txt')
      write(12,10) 'Integrated solar downward fluxes file'
      write(12,10) 'Number of statistical events used:'
      write(12,*) Nevent
      write(12,10) 'File created by program: ',
     &     prog_name(1:strlen(prog_name))
      write(12,10) 'Date of record:',date_str(1:strlen(date_str))
      write(12,10) 'Number of latitude bands:'
      write(12,*) Nphi
      write(12,10) 'Number of longitude bands:'
      write(12,*) Ntheta
      write(12,*)

      do iphi=1,Nphi
         write(12,10) 'Latitude band index:'
         write(12,*) iphi
         write(12,10) 'Minimum phi (rad):'
         write(12,*) phi(iphi-1)
         P(1)=rground
         P(2)=0.0D+0
         P(3)=phi(iphi-1)
         call local_time(dim,P,phi0,time,hour,min,sec)
         call time_string(hour,min,sec,time_str)
         write(12,10) 'Local time (hh:mm:ss) :',time_str
         write(12,10) 'Maximum phi (rad):'
         write(12,*) phi(iphi)
         P(3)=phi(iphi)
         call local_time(dim,P,phi0,time,hour,min,sec)
         call time_string(hour,min,sec,time_str)
         write(12,10) 'Local time (hh:mm:ss) :',time_str
         write(12,*)

         do itheta=1,Ntheta
            write(12,10) 'Longitude band index:'
            write(12,*) itheta
            write(12,10) 'Minimum theta (rad):'
            write(12,*) theta(itheta-1)
            write(12,10) 'Maximum theta (rad):'
            write(12,*) theta(itheta)
            write(12,10) 'Number of vertical interfaces:'
            write(12,*) Nr(itheta,iphi)+1
            write(12,10) 'altitudes of interfaces (m):'
            write(12,50) (z(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(12,10) 'radiuses of interfaces (m):'
            write(12,50) (r(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(12,10) 'Integrated solar downward fluxes (W/m):'
            write(12,50) (solar_flux_dn(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(12,10) 'Uncertainties (W/m):'
            write(12,50) (solar_inc_flux_dn(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
         enddo ! itheta
      enddo ! iphi
      close(12)


c     **** recording integrated solar net fluxes ****
      open(13,file='./results/integrated_solar_net_fluxes.txt')
      write(13,10) 'Integrated solar net fluxes file'
      write(13,10) 'Number of statistical events used:'
      write(13,*) Nevent
      write(13,10) 'File created by program: ',
     &     prog_name(1:strlen(prog_name))
      write(13,10) 'Date of record:',date_str(1:strlen(date_str))
      write(13,10) 'Number of latitude bands:'
      write(13,*) Nphi
      write(13,10) 'Number of longitude bands:'
      write(13,*) Ntheta
      write(13,*)

      do iphi=1,Nphi
         write(13,10) 'Latitude band index:'
         write(13,*) iphi
         write(13,10) 'Minimum phi (rad):'
         write(13,*) phi(iphi-1)
         P(1)=rground
         P(2)=0.0D+0
         P(3)=phi(iphi-1)
         call local_time(dim,P,phi0,time,hour,min,sec)
         call time_string(hour,min,sec,time_str)
         write(13,10) 'Local time (hh:mm:ss) :',time_str
         write(13,10) 'Maximum phi (rad):'
         write(13,*) phi(iphi)
         P(3)=phi(iphi)
         call local_time(dim,P,phi0,time,hour,min,sec)
         call time_string(hour,min,sec,time_str)
         write(13,10) 'Local time (hh:mm:ss) :',time_str
         write(13,*)

         do itheta=1,Ntheta
            write(13,10) 'Longitude band index:'
            write(13,*) itheta
            write(13,10) 'Minimum theta (rad):'
            write(13,*) theta(itheta-1)
            write(13,10) 'Maximum theta (rad):'
            write(13,*) theta(itheta)
            write(13,10) 'Number of vertical interfaces:'
            write(13,*) Nr(itheta,iphi)+1
            write(13,10) 'altitudes of interfaces (m):'
            write(13,50) (z(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(13,10) 'radiuses of interfaces (m):'
            write(13,50) (r(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(13,10) 'Integrated solar net fluxes (W/m):'
            write(13,50) (solar_flux_net(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(13,10) 'Uncertainties (W/m):'
            write(13,50) (solar_inc_flux_net(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
         enddo ! itheta
      enddo ! iphi
      close(13)


      if (toa_ad.eq.1) then
c     **** recording integrated solar TOA fluxes ****
         open(14,file='./results/integrated_solar_TOA_fluxes.txt')
         write(14,10) 'Integrated solar TOA fluxes (W/m)'
         write(14,10) 'Number of statistical events used:'
         write(14,*) Nevent
         write(14,10) 'File created by program: ',
     &        prog_name(1:strlen(prog_name))
         write(14,10) 'Date of record:',date_str(1:strlen(date_str))
         write(14,10) 'Number of latitude bands:'
         write(14,*) Ntheta
         write(14,10) 'Number of longitude bands:'
         write(14,*) Nphi
         write(14,10) 'Latitude discretization:'
         write(14,50) (theta(itheta),itheta=0,Ntheta)
         write(14,10) 'Longitude discretization:'
         write(14,50) (phi(iphi),iphi=0,Nphi)
         write(14,10) 'Number of gas narrowband spectral intervals:'
         write(14,*) Nb_gas
         write(14,10) 'Narrowbands lower limits:'
         write(14,50) (lambda_gas_lo(band),band=1,Nb_gas)
         write(14,10) 'Narrowbands higher limits:'
         write(14,50) (lambda_gas_hi(band),band=1,Nb_gas)
         write(14,10) 'Radius at ground level (m):'
         write(14,50) Radius
         write(14,10) 'Radius at space level (m):'
         write(14,50) rspace
         write(14,*)

         do iphi=1,Nphi
            do itheta=1,Ntheta
               do u_iphi=1,Nphi
                  do u_itheta=1,Ntheta
                     write(14,51) iphi,itheta,u_iphi,u_itheta
                     write(14,50) (solar_flux_toa(itheta,iphi,
     &                    u_itheta,u_iphi,band),band=1,Nb_gas),
     &                    (solar_inc_flux_toa(itheta,iphi,
     &                    u_itheta,u_iphi,band),band=1,Nb_gas)
                  enddo         ! u_itheta
               enddo            ! u_iphi
               write(14,10) 'Sum of TOA fluxes (W/m) / '//
     &              'uncertainty (W/m):'
               write(14,50) sum_solar_flux_toa(itheta,iphi),
     &              sum_inc_solar_flux_toa(itheta,iphi)
               write(14,10) 'Upward flux @TOA (W/m) / '//
     &              'uncertainty (W/m):'
               write(14,50) solar_flux_up(Nr(itheta,iphi),itheta,iphi),
     &              solar_inc_flux_up(Nr(itheta,iphi),itheta,iphi)
            enddo               ! itheta
         enddo                  ! iphi
         close(14)
      endif



c     **** recording integrated solar budgets ****
      open(15,file='./results/integrated_solar_budgets.txt')
      write(15,10) 'Integrated solar budgets file'
      write(15,10) 'Number of statistical events used:'
      write(15,*) Nevent
      write(15,10) 'File created by program: ',
     &     prog_name(1:strlen(prog_name))
      write(15,10) 'Date of record:',date_str(1:strlen(date_str))
      write(15,10) 'Number of latitude bands:'
      write(15,*) Nphi
      write(15,10) 'Number of longitude bands:'
      write(15,*) Ntheta
      write(15,*)

      do iphi=1,Nphi
         write(15,10) 'Latitude band index:'
         write(15,*) iphi
         write(15,10) 'Minimum phi (rad):'
         write(15,*) phi(iphi-1)
         P(1)=rground
         P(2)=0.0D+0
         P(3)=phi(iphi-1)
         call local_time(dim,P,phi0,time,hour,min,sec)
         call time_string(hour,min,sec,time_str)
         write(15,10) 'Local time (hh:mm:ss) :',time_str
         write(15,10) 'Maximum phi (rad):'
         write(15,*) phi(iphi)
         P(3)=phi(iphi)
         call local_time(dim,P,phi0,time,hour,min,sec)
         call time_string(hour,min,sec,time_str)
         write(15,10) 'Local time (hh:mm:ss) :',time_str
         write(15,*)

         do itheta=1,Ntheta
            write(15,10) 'Longitude band index:'
            write(15,*) itheta
            write(15,10) 'Minimum theta (rad):'
            write(15,*) theta(itheta-1)
            write(15,10) 'Maximum theta (rad):'
            write(15,*) theta(itheta)
            write(15,10) 'Number of vertical interfaces:'
            write(15,*) Nr(itheta,iphi)+1
            write(15,10) 'altitudes of interfaces (m):'
            write(15,50) (z(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(15,10) 'radiuses of interfaces (m):'
            write(15,50) (r(ir,itheta,iphi),
     &           ir=0,Nr(itheta,iphi))
            write(15,10) 'Integrated solar budget for ground (W/m):'
            write(15,50) solar_budget(0,itheta,iphi)
            write(15,10) 'Uncertainty (W/m):'
            write(15,50) solar_inc_budget(0,itheta,iphi)
            write(15,10) 'Integrated solar budget for gas cells (W/m):'
            write(15,50) (solar_budget(ir,itheta,iphi),
     &           ir=1,Nr(itheta,iphi))
            write(15,10) 'Uncertainties (W/m):'
            write(15,50) (solar_inc_budget(ir,itheta,iphi),
     &           ir=1,Nr(itheta,iphi))
            write(15,10) 'Integrated solar budget for space (W/m):'
            write(15,50) solar_budget(Nr(itheta,iphi),itheta,iphi)
            write(15,10) 'Uncertainty (W/m):'
            write(15,50) solar_inc_budget(Nr(itheta,iphi),itheta,iphi)
         enddo ! itheta
      enddo ! iphi
      close(15)

      return
      end




      subroutine local_time(dim,P,phi0,time,hour,min,sec)
      implicit none
      include 'max.inc'
      include 'param.inc'
c     
c     Purpose: to compute the local time at a given position "P"
c
c     Inputs:
c       + dim: dimension of vectors
c       + P: (r,theta,phi) spherical coordinates of the position on the planet
c       + phi0: longitude of the point where the star is at the zenith (rad)
c
c     Outputs:
c       + time: time in hours, on a 24h format (i.e. 10.25=10h15min)
c       + hour: hour (0-23)
c       + min: minute (0-59)
c       + sec: seconds (0-59)
c

c     input
      integer dim
      double precision P(1:Nmat_mx)
      double precision phi0
c     output
      double precision time
      integer hour,min,sec
c     temp
      integer strlen
      character*100 label
      label='subroutine local_time'

      call test_phi(P(3),label)
      call test_phi(phi0,label)

      time=12.0D+0+(P(3)-phi0)/pi
      if (time.ge.24.0D+0) then
         time=time-24.0D+0
      endif

      call convert_time(time,hour,min,sec)

      return
      end



      subroutine convert_time(time,hour,min,sec)
      implicit none
c     
c     Purpose: to convert 'time' into human-compatible format
c
c     Inputs:
c       + time: time on 24h format (i.e. 10.25=10h15min)
c     
c     Outputs:
c       + hour: hour (0-23)
c       + min: minute (0-59)
c       + sec: seconds (0-59)
c       

c     input
      double precision time
c     output
      integer hour,min,sec
c     temp
      integer nspm,nmph,nhpd
      double precision m,s
      integer strlen
      character*100 label
      label='subroutine convert_time'

      nspm=60
      nmph=60
      nhpd=24

      if (time.ge.dble(nhpd)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'time=',time
         write(*,*) 'is greater than 24h !'
         stop
      endif

      hour=int(time)
      m=time-dble(hour)
      min=int(m*dble(nmph))
      s=time-dble(hour)-dble(min)/dble(nmph)
      sec=int(s*dble(nmph*nspm))

      return
      end



      subroutine save_logs(Nevent)
      implicit none
      include 'mpif.h'
      include 'formats.inc'
c
c     Purpose: to save log files into a unique folder
c

c     input
      integer Nevent
c     MPICH
      integer ierr,errorcode,np,pindex
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     temp
      integer ok,appendix,appendix_max
      logical ex
      character*10 np_ch,N_ch,core_ch,app_ch
      character*100 date_ch
      character*100 path,folder,folder_path,f
      character*100 hostname
      character*200 command
      integer year,month,day
      integer strlen
      character*100 label
c     MPICH
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH
      label='subroutine save_logs'
      appendix_max=100

c     get the host name
c      command='echo $HOSTNAME > ./hostname'
      command='./bash/write_hostname.bash'
      call exec(command)
      open(10,file='./hostname')
      read(10,10) hostname
      close(10)
c     get the number of cores used
      call num2str(np-1,np_ch)
      if (np-1.eq.1) then
         core_ch='core'
      else
         core_ch='cores'
      endif
c     get the number of statistical event
      call num2str(Nevent,N_ch)
c     get the date
      call get_date(year,month,day)
      call date_string(year,month,day,date_ch)
c     set the main path
      path='./logs/archive/'
c     now concatenate base folder name
      folder=hostname(1:strlen(hostname))//'_'//
     &     np_ch(1:strlen(np_ch))//core_ch(1:strlen(core_ch))//'_'//
     &     date_ch(1:strlen(date_ch))//'_'//
     &     N_ch(1:strlen(N_ch))//'events'
c     temporary folder name "f"
      f=folder
      ok=0

      appendix=0
      do while (ok.eq.0)
         folder_path=path(1:strlen(path))
     &        //f(1:strlen(f))
     &        //'/seed'
         inquire(file=folder_path,exist=ex)
         if (ex) then
            ok=0
            appendix=appendix+1
            if (appendix.gt.appendix_max) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'maximal number of folders reached'
               stop
            endif
            call num2str(appendix,app_ch)
            f=folder(1:strlen(folder))//
     &           '_'//app_ch(1:strlen(app_ch))
         else
            ok=1
         endif
      enddo
c     path to final path
      folder_path=path(1:strlen(path))//f(1:strlen(f))
c     create final folder
      command='cd ./logs/archive; mkdir '//f(1:strlen(f))
      call exec(command)
c     copy log files into final folder
      command='cp ./config/seed '//
     &     folder_path(1:strlen(folder_path))
      call exec(command)
      command='cp ./logs/*.log '//
     &     folder_path(1:strlen(folder_path))//
     &     ' 2>/dev/null'
      call exec(command)

      return
      end



      subroutine get_date(year,month,day)
      implicit none
      include 'formats.inc'
c
c     Purpose: to compute current year, month and day indexes
c     Please note this routine needs command "date +%D" to be run on the system
c     
c     Outputs:
c       + year: index of the current year
c       + month: index of the current month
c       + day: index of the current day
c

c     outputs
      integer year,month,day
c     temp
      integer i,j,first,firstf,last,lastf
      character*1 sp_ch,ch,s_ch
      character*10 str,year_ch,month_ch,day_ch
      character*100 date_str
      character*200 command
      character*100 label
      integer strlen
      label='subroutine get_date'
      
      command='date +%D > ./date'
      call exec(command)
      open(10,file='./date')
      read(10,10) date_str
      close(10)

c     Debug
c      write(*,*) date_str(1:strlen(date_str))
c     Debug

      write(sp_ch,10) ' '
      s_ch='/'
      firstf=0
      do i=1,strlen(date_str)
         ch=date_str(i:i+1)
c     Debug
c         write(*,*) 'ch="',ch(1:strlen(ch)),'"'
c     Debug
         if (ch(1:strlen(ch)).ne.sp_ch(1:strlen(sp_ch))) then
            firstf=1
            first=i
            goto 111
         endif
      enddo
 111  continue
c     Debug
c      write(*,*) 'firstf=',first
c      if (firstf.eq.1) then
c         write(*,*) 'first=',first
c      endif
c     Debug
      
      do j=1,3
c     Debug
c         write(*,*) 'first=',first
c     Debug
         lastf=0
         do i=first,strlen(date_str)
            ch=date_str(i:i+1)
            if (ch(1:strlen(ch)).eq.s_ch(1:strlen(s_ch))) then
               lastf=1
               last=i
               goto 222
            endif
         enddo
 222     continue
         if ((lastf.eq.0).and.(i.eq.strlen(date_str)+1)) then
            lastf=1
            last=strlen(date_str)+1
         endif
         if (lastf.eq.0) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'i=',i,strlen(date_str)
            write(*,*) 'The "/" character was not found'
            stop
         endif
c     Debug
c      write(*,*) 'lastf=',lastf
c      if (lastf.eq.1) then
c         write(*,*) 'last=',last
c      endif
c     Debug
         str=date_str(first:last-1)
c     Debug
c         write(*,*) 'str=',str(1:strlen(str))
c     Debug
         if (j.eq.1) then
            call str2num(str,month)
c     Debug
c            write(*,*) 'month=',month
c     Debug
         else if (j.eq.2) then
            call str2num(str,day)
c     Debug
c            write(*,*) 'day=',day
c     Debug
         else if (j.eq.3) then
            call str2num(str,year)
c     Debug
c            write(*,*) 'year=',year
c     Debug
            year=year+2000
         endif
         first=last+1
      enddo ! j

      return
      end




      subroutine date_string(year,month,day,date_str)
      implicit none
c     
c     Purpose: to concatenate a date string from year, month and day indexes
c
c     Inputs:
c       + year: index of the current year
c       + month: index of the current month
c       + day: index of the current day
c
c     Output:
c       + date_str: date string
c

c     inputs
      integer year,month,day
c     output
      character*100 date_str
c     temp
      character*10 year_ch,month_ch,day_ch
      integer strlen
      character*100 label
      label='subroutine date_string'

      call num2str(year,year_ch)
      call num2str(month,month_ch)
      call num2str(day,day_ch)
      date_str=year_ch(1:strlen(year_ch))//'-'//
     &     month_ch(1:strlen(month_ch))//'-'//
     &     day_ch(1:strlen(day_ch))

      return
      end




      subroutine time_string(hour,min,sec,time_str)
      implicit none
c     
c     Purpose: to concatenate a date string from hour, minute and second indexes
c
c     Inputs:
c       + hour: hour index
c       + min: minute index
c       + sec: second index
c
c     Output:
c       + time_str: date string
c

c     inputs
      integer hour,min,sec
c     output
      character*100 time_str
c     temp
      character*10 hour_ch,min_ch,sec_ch
      integer strlen
      character*100 label
      label='subroutine time_string'

      call num2str(hour,hour_ch)
      call num2str(min,min_ch)
      call num2str(sec,sec_ch)
      time_str=hour_ch(1:strlen(hour_ch))//':'//
     &     min_ch(1:strlen(min_ch))//':'//
     &     sec_ch(1:strlen(sec_ch))

      return
      end


      subroutine clean()
      implicit none
c
c     Purpose: to clean files that have been temporarily used by the code
c
      character*200 command
      integer strlen
      character*100 label
      label='subroutine clean'

      command='rm -f date'
      call exec(command)
      command='rm -f hostname'
      call exec(command)
      command='rm -f nlines_*'
      call exec(command)

      return
      end
