c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine random_gen(tmp)
      implicit none
c
c     Main pseudo-random number generator routine
c     (the one that must be called)
c
      integer n,t
      double precision tmp

      n=1
      t=0
 10   continue
      call zufall(n,tmp)
      if (tmp.eq.0.or.tmp.eq.1) then
         t=t+1
         if (t.gt.10) then
            write(*,*) 'Error from random_gen:'
            write(*,*) 'Too many zeroes'
            stop
         endif
         goto 10
      endif

      return
      end


      subroutine uniform(val1,val2,alpha)
      implicit none
      include 'param.inc'
      include 'max.inc'
c
c     Purpose: to return a value between "val1" and "val2", chosen uniformally
c
      double precision R,val1,val2,alpha

      if (val2.le.val1) then
         write(*,*) 'Error from uniform:'
         write(*,*) 'val2=',val2
         write(*,*) '< val1=',val1
         stop
      endif
      call random_gen(R)
      alpha=val1+(val2-val1)*R

      return
      end


      subroutine choose_integer(N1,N2,i)
      implicit none
c
c     Purpose: to choose an integer between N1 and N2 (included)
c     on a random uniform basis
c
c     Inputs:
c       + N1: minimum value the integer can take
c       + N2: maximum value the integer can take
c
c     Output:
c       + i: integer value between N1 and N2
c
      integer N1,N2,i
      double precision R

      call random_gen(R)
      i=N1+int((N2-N1)*R)

      return
      end


      subroutine cross_section(dim,R,theta0,phi0,P0,pdf)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to choose a position "P0" on the planetary cross-section
c     that is defined by a total external radius "R" (at the top of the atmosphere)
c     and that is perpendicular to solar input direction u0 defined
c     by angles "theta0" and "phi0"
c
c     Inputs:
c       + dim: dimension of vectors
c       + R: radius of the planet cross-section
c       + theta0: latitude of the point where the star is at the zenith (rad)
c       + phi0: longitude of the point where the star is at the zenith (rad)
c
c     Ouputs:
c       + P0: (r,theta,phi) spherical coordinates of the position
c            that was chosen on the planet cross-section
c       + pdf: pdf associated to the choice of position "P0" over the "pi*R" surface (m)
c

c     inputs
      integer dim
      double precision R,theta0,phi0
c     output
      double precision P0(1:Nmat_mx)
      double precision pdf
c     temp
      integer strlen,i
      double precision radius,alpha
      double precision u1(1:Nmat_mx)
      double precision u2(1:Nmat_mx)
      double precision u3(1:Nmat_mx)
      double precision axe1(1:Nmat_mx)
      double precision axe2(1:Nmat_mx)
      double precision P0_cart(1:Nmat_mx)
      double precision rand
      character*100 label
      label='subroutine cross_section'

      call random_gen(rand)
      radius=R*dsqrt(rand)
      call uniform(0.0D+0,2.0D+0*pi,alpha)
c     "u1" is the direction between the origin and the point chosen on a
c     first disk "D1" of radius "R" and contained in plane x=0
      u1(1)=0.0D+0
      u1(2)=dcos(alpha)
      u1(3)=dsin(alpha)
c     disk "D1" becomes disk "D2" after a rotation of angle "phi0"
c     around the z-axis
      axe1(1)=0.0D+0
      axe1(2)=0.0D+0
      axe1(3)=1.0D+0
      call rotation(dim,u1,phi0,axe1,u2)
c     disk "D2" becomes disk "D3" after a rotation of angle "theta0"
c     around the horizontal axis defined by direction "phi0"
      axe2(1)=-dsin(phi0)
      axe2(2)=dcos(phi0)
      axe2(3)=0.0D+0
      call rotation(dim,u2,-theta0,axe2,u3)
c     then coordinates of point "P0" can be computed from direction "u3"
      call scalar_vector(dim,radius,u3,P0_cart)
      call cart2spher(P0_cart(1),P0_cart(2),P0_cart(3),
     &     P0(1),P0(2),P0(3))
c      pdf=2.0D+0/3.0D+0*1.0D+0/(pi*R**2.0D+0)
      pdf=1.0D+0/(pi*R**2.0D+0) ! m
      
      return
      end



      subroutine half_sphere_uniform(dim,R,theta0,phi0,P,pdf)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to choose a position "P" on the planetary half-sphere
c     that is exposed to sunlight, defined by a total external radius "R"
c     (at the top of the atmosphere)
c
c     Inputs:
c       + dim: dimension of vectors
c       + R: radius of the planet cross-section
c       + theta0: latitude of the point where the star is at the zenith (rad)
c       + phi0: longitude of the point where the star is at the zenith (rad)
c
c     Ouputs:
c       + P: (r,theta,phi) spherical coordinates of the position
c            that was chosen on the surface exposed to sunlight
c       + pdf: pdf associated to the choice of position "P" over the "2*pi*R" surface
c

c     inputs
      integer dim
      double precision R,theta0,phi0
c     output
      double precision P(1:Nmat_mx)
      double precision pdf
c     temp
      integer strlen,i
      double precision rand
      double precision P1(1:Nmat_mx)
      double precision u1(1:Nmat_mx)
      double precision u2(1:Nmat_mx)
      double precision u3(1:Nmat_mx)
      double precision axe1(1:Nmat_mx)
      double precision axe2(1:Nmat_mx)
      double precision P0_cart(1:Nmat_mx)
      character*100 label
      label='subroutine half_sphere_uniform'

      P1(1)=1.0D+0
      call random_gen(rand)
      P1(2)=dasin(2.0D+0*rand-1.0D+0)
      call random_gen(rand)
      P1(3)=pi*(rand-0.5D+0)
      if (P1(3).lt.0.0D+0) then
         P1(3)=P1(3)+2.0D+0*pi
      endif
c     checking
      if ((P1(2).lt.-pi/2.0D+0).or.(P1(2).gt.pi/2.0D+0)) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'P1(2)=',P1(2)
         stop
      endif
      if ((P1(3).lt.0.0D+0).or.(P1(3).gt.pi*2.0D+0)) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'P1(3)=',P1(3)
         stop
      endif
      call spher2cart(label,1,P1(1),P1(2),P1(3),u1(1),u1(2),u1(3))

c     disk "D1" becomes disk "D2" after a rotation of angle "phi0"
c     around the z-axis
      axe1(1)=0.0D+0
      axe1(2)=0.0D+0
      axe1(3)=1.0D+0
      call rotation(dim,u1,phi0,axe1,u2)
c     disk "D2" becomes disk "D3" after a rotation of angle "theta0"
c     around the horizontal axis defined by direction "phi0"
      axe2(1)=-dsin(phi0)
      axe2(2)=dcos(phi0)
      axe2(3)=0.0D+0
      call rotation(dim,u2,-theta0,axe2,u3)
c     then coordinates of point "P" can be computed from direction "u3"
      call scalar_vector(dim,R,u3,P0_cart)
      call cart2spher(P0_cart(1),P0_cart(2),P0_cart(3),
     &     P(1),P(2),P(3))
      pdf=1.0D+0/(2.0D+0*pi*R**2.0D+0)
c      pdf=1.0D+0

      return
      end
