c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine rotation(dim,u,alpha,axe,v)
      implicit none
      include 'param.inc'
      include 'max.inc'
c
c     Purpose: to compute coordinates of vector "v", the transform of vector "u"
c     by a rotation of angle "alpha" around and axe
c
c     Inputs:
c       + dim: dimension of vectors
c       + u: (ux,uy,uz) coordinates of vector "u"
c       + alpha: rotation angle
c       + axe: (ax,ay,az) coordinates of the elementary vector definig the direction of the rotation axe
c     
c     Outputs:
c       + v: (vx,vy,vz) coordinates of vector "v"
c

c     inputs
      integer dim
      double precision u(1:Nmat_mx)
      double precision alpha
      double precision axe(1:Nmat_mx)
c     ouputs
      double precision v(1:Nmat_mx)
c     temp
      double precision scalar
      double precision naxe(1:Nmat_mx)
      double precision M(1:Nmat_mx,1:Nmat_mx)
      double precision I3(1:Nmat_mx,1:Nmat_mx)
      double precision Maxe1(1:Nmat_mx,1:Nmat_mx)
      double precision Maxe2(1:Nmat_mx,1:Nmat_mx)
      double precision Mt1(1:Nmat_mx,1:Nmat_mx)
      double precision Mt2(1:Nmat_mx,1:Nmat_mx)
      double precision Mt3(1:Nmat_mx,1:Nmat_mx)
      double precision Mt4(1:Nmat_mx,1:Nmat_mx)

c     checking inputs are understood
      call normalize_vector(dim,axe,naxe)
      call axe_matrix(naxe,Maxe1,Maxe2)
c     computation of rotation matrix M
      call identity_matrix(dim,I3)
      scalar=dcos(alpha)
      call matrix_scalar(dim,I3,scalar,Mt1)
      scalar=1.0D+0-dcos(alpha)
      call matrix_scalar(dim,Maxe1,scalar,Mt2)
      scalar=dsin(alpha)
      call matrix_scalar(dim,Maxe2,scalar,Mt3)
      call add_matrix(dim,Mt1,Mt2,Mt4)
      call add_matrix(dim,Mt4,Mt3,M)
c     computation of vector "v" coordinates
      call matrix_vector(dim,M,u,v)

      return
      end



      subroutine axe_matrix(axe,Maxe1,Maxe2)
      implicit none
      include 'max.inc'
      double precision axe(1:Nmat_mx)
      double precision Maxe1(1:Nmat_mx,1:Nmat_mx)
      double precision Maxe2(1:Nmat_mx,1:Nmat_mx)

      Maxe1(1,1)=axe(1)**2.0D+0
      Maxe1(1,2)=axe(1)*axe(2)
      Maxe1(1,3)=axe(1)*axe(3)
      Maxe1(2,1)=axe(1)*axe(2)
      Maxe1(2,2)=axe(2)**2.0D+0
      Maxe1(2,3)=axe(2)*axe(3)
      Maxe1(3,1)=axe(1)*axe(3)
      Maxe1(3,2)=axe(2)*axe(3)
      Maxe1(3,3)=axe(3)**2.0D+0

      Maxe2(1,1)=0.0D+0
      Maxe2(1,2)=-axe(3)
      Maxe2(1,3)=axe(2)
      Maxe2(2,1)=axe(3)
      Maxe2(2,2)=0.0D+0
      Maxe2(2,3)=-axe(1)
      Maxe2(3,1)=-axe(2)
      Maxe2(3,2)=axe(1)
      Maxe2(3,3)=0.0D+0

      return
      end
